<%inherit file="base.html" />
<%namespace name='static' file='static_content.html'/>
<%!
  from openedx.core.djangolib.js_utils import dump_js_escaped_json, js_escaped_string
%>
<%block name="title">${"Course management"}</%block>
<%block name="header_extras">

</%block>

<%block name="content">
    <link type='text/css' rel='stylesheet' href="${static.url('bundle/css/course-management.bundle.css')}">
    <div class="wrapper-content wrapper content">
        <h1>Course Management</h1>
        <div id="root"></div>
        <script src="${static.url('bundle/js/build.js')}"></script>
        <script>
            new RenderReactComponent({
                courseSettings: {
                    course_key: "${course_key | n, js_escaped_string}",
                    details_fields: ${details_fields | n, dump_js_escaped_json},
                    request_timeout_value: ${request_timeout_value | n, dump_js_escaped_json}
                },
                courseRerun: {
                    request_timeout_value: ${request_timeout_value | n, dump_js_escaped_json}
                }
            });
        </script>
    </div>
</%block>
